@@ -1,10 +1,12 @@
 from codepack import Code, Callback, StorageService
 from codepack.storages import MongoStorage
-from tests import *
+from tests import add2, add3, mul2, print_x, combination, linear,\
+    dummy_function1, dummy_function2, dummy_callback1, dummy_callback2, dummy_callback3
 import pytest
 from datetime import datetime
 from unittest.mock import MagicMock
 from functools import partial
+import codepack.utils.functions
 
 
 def test_assert_param(default_os_env):
@@ -68,24 +70,24 @@ def test_print_info(default_os_env):
     code2 = Code(add3, image='test-image', owner='admin')
     code1 >> code2
     code2.receive('b') << code1
-    assert code1.get_info() == "Code(id: add2, function: add2, params: (a, b), receive: {}," \
+    assert code1.get_info() == "Code(id: add2, function: add2, params: (a, b)," \
                                " env: test-env, state: UNKNOWN)"
     assert code2.get_info() == "Code(id: add3, function: add3, params: (a, b, c=2), receive: {'b': 'add2'}," \
                                " image: test-image, owner: admin, state: UNKNOWN)"
-    assert code1.get_info(state=False) == "Code(id: add2, function: add2, params: (a, b), receive: {}, env: test-env)"
-    assert code2.get_info(state=False) == "Code(id: add3, function: add3, params: (a, b, c=2), receive: {'b': 'add2'}," \
-                                          " image: test-image, owner: admin)"
+    assert code1.get_info(state=False) == "Code(id: add2, function: add2, params: (a, b), env: test-env)"
+    assert code2.get_info(state=False) == "Code(id: add3, function: add3, params: (a, b, c=2)," \
+                                          " receive: {'b': 'add2'}, image: test-image, owner: admin)"
     code1.image = 'test-image2'
     code1.owner = 'admin2'
-    assert code1.get_info() == "Code(id: add2, function: add2, params: (a, b), receive: {}," \
+    assert code1.get_info() == "Code(id: add2, function: add2, params: (a, b)," \
                                " env: test-env, image: test-image2, owner: admin2, state: UNKNOWN)"
-    assert code1.get_info(state=False) == "Code(id: add2, function: add2, params: (a, b), receive: {}," \
+    assert code1.get_info(state=False) == "Code(id: add2, function: add2, params: (a, b)," \
                                           " env: test-env, image: test-image2, owner: admin2)"
     code2.owner = None
     assert code2.get_info() == "Code(id: add3, function: add3, params: (a, b, c=2), receive: {'b': 'add2'}," \
                                " image: test-image, state: UNKNOWN)"
-    assert code2.get_info(state=False) == "Code(id: add3, function: add3, params: (a, b, c=2), receive: {'b': 'add2'}," \
-                                          " image: test-image)"
+    assert code2.get_info(state=False) == "Code(id: add3, function: add3, params: (a, b, c=2)," \
+                                          " receive: {'b': 'add2'}, image: test-image)"
     code2.image = None
     assert code2.get_info() == "Code(id: add3, function: add3, params: (a, b, c=2), receive: {'b': 'add2'}," \
                                " state: UNKNOWN)"
@@ -93,6 +95,13 @@ def test_print_info(default_os_env):
         state=False) == "Code(id: add3, function: add3, params: (a, b, c=2), receive: {'b': 'add2'})"
 
 
+def test_partial_print_info(default_os_env):
+    plus2 = partial(add2, b=3)
+    code = Code(plus2, owner='admin')
+    assert code.get_info() == "Code(id: add2, function: add2, params: (a, b)," \
+                              " context: {'b': 3}, owner: admin, state: UNKNOWN)"
+
+
 def test_get_function_from_source(default_os_env):
     source = """def plus1(x):\n  return x + 1"""
     code = Code(source=source)
@@ -237,7 +246,7 @@ def test_validate_dependency_result(default_os_env):
     code2(3, 4)
     code2.send_result(item=1, timestamp=datetime.now().timestamp() + 1)
     ret = code3(a=3)
-    assert ret is 7
+    assert ret == 7
     assert code3.get_state() == 'TERMINATED'
     code2(3, 4)
     snapshots = code3.dependency.load_snapshot()
@@ -272,16 +281,6 @@ def test_default_arg(default_os_env):
     assert ret == 6 and code2.get_state() == 'TERMINATED'
 
 
-def test_load_code_from_storage_service_with_id(default_os_env):
-    code1 = Code(add2)
-    code1.save()
-    code2 = Code(id='add2', serial_number='1234')
-    assert code1.source == code2.source
-    assert code1(1, 2) == code2(1, 2)
-    assert code1.serial_number != code2.serial_number
-    assert code2.serial_number == '1234'
-
-
 def test_update_serial_number(default_os_env):
     code1 = Code(add2)
     code2 = Code(mul2)
@@ -357,7 +356,7 @@ def test_recursion_detection(default_os_env):
 
 
 def test_default_load(default_os_env):
-    code1 = Code(add2)
+    _ = Code(add2)
     code2 = Code(add3)
     search_result = Code.load(['add2', 'add3'])
     assert type(search_result) == list and len(search_result) == 0
@@ -412,14 +411,17 @@ def test_remove(default_os_env):
 def test_register_callback_with_names(default_os_env):
     code = Code(add2)
     code.register_callback(callback=dummy_callback1, name=['callback1', 'callback2', 'callback3'])
-    assert code.callbacks == {'callback1': dummy_callback1}
+    assert len(code.callback) == 1
+    assert 'callback1' in code.callback
+    assert isinstance(code.callback['callback1'], Callback)
+    assert code.callback['callback1'].to_dict() == Callback(dummy_callback1).to_dict()
 
 
 def test_register_multiple_callbacks_without_names(default_os_env):
     code = Code(add2)
     code.register_callback(callback=[dummy_callback1, partial(dummy_callback2, 'test'), Callback(dummy_callback3)])
-    assert len(code.callbacks) == 3
-    assert set(code.callbacks.keys()) == {'dummy_callback1', 'dummy_callback2', 'dummy_callback3'}
+    assert len(code.callback) == 3
+    assert set(code.callback.keys()) == {'dummy_callback1', 'dummy_callback2', 'dummy_callback3'}
 
 
 def test_register_multiple_callbacks_with_one_name(default_os_env):
@@ -427,11 +429,12 @@ def test_register_multiple_callbacks_with_one_name(default_os_env):
     with pytest.raises(IndexError):
         code.register_callback(callback=[dummy_callback1, partial(dummy_callback2, 'test'), Callback(dummy_callback3)],
                                name='dummy_callback')
-    assert len(code.callbacks) == 0
+    assert len(code.callback) == 0
     code.register_callback(callback=[dummy_callback1], name='dummy_callback')
-    assert len(code.callbacks) == 1
-    assert 'dummy_callback' in code.callbacks
-    assert code.callbacks['dummy_callback'] == dummy_callback1
+    assert len(code.callback) == 1
+    assert 'dummy_callback' in code.callback
+    assert isinstance(code.callback['dummy_callback'], Callback)
+    assert code.callback['dummy_callback'].to_dict() == Callback(dummy_callback1).to_dict()
 
 
 def test_register_multiple_callbacks_with_multiple_names(default_os_env):
@@ -441,17 +444,22 @@ def test_register_multiple_callbacks_with_multiple_names(default_os_env):
     with pytest.raises(IndexError):
         code.register_callback(callback=[dummy_callback1, _dummy_callback2, _dummy_callback3],
                                name=['dummy_callback'])
-    assert len(code.callbacks) == 0
+    assert len(code.callback) == 0
     code.register_callback(callback=[dummy_callback1, _dummy_callback2, _dummy_callback3],
                            name=['callback1', 'callback2', 'callback3'])
-    assert len(code.callbacks) == 3
-    assert code.callbacks == {'callback1': dummy_callback1,
-                              'callback2': _dummy_callback2,
-                              'callback3': _dummy_callback3}
+    assert len(code.callback) == 3
+    for callback_name in ['callback1', 'callback2', 'callback3']:
+        assert callback_name in code.callback
+        assert isinstance(code.callback[callback_name], Callback)
+    assert code.callback['callback1'].to_dict() == Callback(dummy_callback1).to_dict()
+    assert code.callback['callback2'].to_dict() == Callback(_dummy_callback2).to_dict()
+    assert code.callback['callback3'] == _dummy_callback3
 
 
 def test_run_callback_with_normal_case(default_os_env):
     mock_function = MagicMock()
+    mock_function.__name__ = 'dummy_name'
+    mock_function.__code__ = MagicMock()
     code = Code(add2)
     code.register_callback(callback=mock_function, name='test_function')
     mock_function.assert_not_called()
@@ -466,6 +474,8 @@ def test_run_callback_with_normal_case(default_os_env):
 
 def test_run_callback_with_error_case(default_os_env):
     mock_function = MagicMock()
+    mock_function.__name__ = 'dummy_name'
+    mock_function.__code__ = MagicMock()
     code = Code(add2)
     code.register_callback(callback=mock_function, name='test_function')
     mock_function.assert_not_called()
@@ -485,6 +495,10 @@ def test_run_callback_with_error_case(default_os_env):
 def test_run_multiple_callbacks_with_normal_case(default_os_env):
     mock_function1 = MagicMock()
     mock_function2 = MagicMock()
+    mock_function1.__name__ = 'dummy_name1'
+    mock_function2.__name__ = 'dummy_name2'
+    mock_function1.__code__ = MagicMock()
+    mock_function2.__code__ = MagicMock()
     code = Code(add2)
     code.register_callback(callback=[mock_function1, mock_function2], name=['test_function1', 'test_function2'])
     mock_function1.assert_not_called()
@@ -502,6 +516,10 @@ def test_run_multiple_callbacks_with_normal_case(default_os_env):
 def test_run_multiple_callbacks_with_error_case(default_os_env):
     mock_function1 = MagicMock()
     mock_function2 = MagicMock()
+    mock_function1.__name__ = 'dummy_name1'
+    mock_function2.__name__ = 'dummy_name2'
+    mock_function1.__code__ = MagicMock()
+    mock_function2.__code__ = MagicMock()
     code = Code(add2)
     code.register_callback(callback=[mock_function1, mock_function2], name=['test_function1', 'test_function2'])
     mock_function1.assert_not_called()
@@ -518,3 +536,28 @@ def test_run_multiple_callbacks_with_error_case(default_os_env):
             if seq[i] == 'ERROR':
                 x['message'] = "add2() missing 1 required positional argument: 'b'"
             assert args[0] == x
+
+
+def test_get_source_of_function_defined_in_string():
+    mock_function1 = MagicMock()
+    mock_function1.__name__ = 'dummy_name'
+    mock_function1.__code__ = MagicMock()
+    mock_function1.__code__.co_filename = '<string>'
+    with pytest.raises(AssertionError):
+        codepack.utils.functions.get_source(mock_function1)
+
+
+def test_partial_code_run(default_os_env):
+    plus1 = partial(add2, b=1)
+    code = Code(plus1)
+    assert code.id == 'add2'
+    assert code.context == {'b': 1}
+    assert code(3) == 4
+    assert code(4, b=5) == 9
+    with pytest.raises(TypeError):
+        code(4, 5)
+    snapshot = code.to_snapshot()
+    code2 = Code.from_snapshot(snapshot)
+    assert code2.id == 'add2'
+    assert code2.context == {'b': 1}
+    assert code2(4) == 5
